@@ -27,6 +27,7 @@
 #include "oobs-object.h"
 #include "oobs-object-private.h"
 #include "oobs-timeconfig.h"
+#include "utils.h"
 
 #define TIME_CONFIG_REMOTE_OBJECT "TimeConfig"
 #define OOBS_TIME_CONFIG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), OOBS_TYPE_TIME_CONFIG, OobsTimeConfigPrivate))
@@ -108,6 +109,7 @@ oobs_time_config_init (OobsTimeConfig *config)
   priv = OOBS_TIME_CONFIG_GET_PRIVATE (config);
 
   priv->time_is_set = FALSE;
+  config->_priv = priv;
 }
 
 static void
@@ -117,7 +119,7 @@ oobs_time_config_finalize (GObject *object)
 
   g_return_if_fail (OOBS_IS_TIME_CONFIG (object));
 
-  priv = OOBS_TIME_CONFIG_GET_PRIVATE (object);
+  priv = OOBS_TIME_CONFIG (object)->_priv;
 
   if (priv && priv->timezone)
     g_free (priv->timezone);
@@ -136,7 +138,7 @@ oobs_time_config_set_property (GObject      *object,
 
   g_return_if_fail (OOBS_IS_TIME_CONFIG (object));
 
-  priv = OOBS_TIME_CONFIG_GET_PRIVATE (object);
+  priv = OOBS_TIME_CONFIG (object)->_priv;
 
   switch (prop_id)
     {
@@ -163,7 +165,7 @@ oobs_time_config_get_property (GObject    *object,
 
   g_return_if_fail (OOBS_IS_TIME_CONFIG (object));
 
-  priv = OOBS_TIME_CONFIG_GET_PRIVATE (object);
+  priv = OOBS_TIME_CONFIG (object)->_priv;
 
   switch (prop_id)
     {
@@ -199,9 +201,9 @@ oobs_time_config_update (OobsObject *object)
   OobsTimeConfigPrivate *priv;
   DBusMessage *reply;
   DBusMessageIter iter;
-  gchar *timezone;
+  const gchar *timezone;
 
-  priv  = OOBS_TIME_CONFIG_GET_PRIVATE (object);
+  priv  = OOBS_TIME_CONFIG (object)->_priv;
   reply = _oobs_object_get_dbus_message (object);
 
   /* First of all, free the previous config */
@@ -219,8 +221,8 @@ oobs_time_config_update (OobsObject *object)
   dbus_message_iter_next (&iter);
 
   dbus_message_iter_next (&iter);
-  dbus_message_iter_get_basic (&iter, &timezone);
-  priv->timezone = g_strdup (timezone);
+  timezone = utils_get_string (&iter);
+  priv->timezone = (timezone) ? g_strdup (timezone) : NULL;
 }
 
 static void
@@ -231,7 +233,7 @@ oobs_time_config_commit (OobsObject *object)
   DBusMessageIter iter;
   gint year, month, day, hour, minute, second;
 
-  priv = OOBS_TIME_CONFIG_GET_PRIVATE (object);
+  priv = OOBS_TIME_CONFIG (object)->_priv;
   message = _oobs_object_get_dbus_message (object);
 
   oobs_time_config_get_utc_time (OOBS_TIME_CONFIG (object),
@@ -400,12 +402,12 @@ real_get_time (OobsTimeConfig *config,
 /**
  * oobs_time_config_get_time:
  * @config: An #OobsTimeConfig.
- * @year: gint pointer to store the year, or NULL.
- * @month: gint pointer to store the month, or NULL.
- * @day: gint pointer to store the day, or NULL.
- * @hour: gint pointer to store the hour, or NULL.
- * @minute: gint pointer to store the minute, or NULL.
- * @second: gint pointer to store the second, or NULL.
+ * @year: gint pointer to store the year, or #NULL.
+ * @month: gint pointer to store the month, or #NULL.
+ * @day: gint pointer to store the day, or #NULL.
+ * @hour: gint pointer to store the hour, or #NULL.
+ * @minute: gint pointer to store the minute, or #NULL.
+ * @second: gint pointer to store the second, or #NULL.
  * 
  * Gets the system time and date in human readable values.
  **/
@@ -426,12 +428,12 @@ oobs_time_config_get_time (OobsTimeConfig *config,
 /**
  * oobs_time_config_get_utc_time:
  * @config: An #OobsTimeConfig.
- * @year: gint pointer to store the year, or NULL.
- * @month: gint pointer to store the month, or NULL.
- * @day: gint pointer to store the day, or NULL.
- * @hour: gint pointer to store the hour, or NULL.
- * @minute: gint pointer to store the minute, or NULL.
- * @second: gint pointer to store the second, or NULL.
+ * @year: gint pointer to store the year, or #NULL.
+ * @month: gint pointer to store the month, or #NULL.
+ * @day: gint pointer to store the day, or #NULL.
+ * @hour: gint pointer to store the hour, or #NULL.
+ * @minute: gint pointer to store the minute, or #NULL.
+ * @second: gint pointer to store the second, or #NULL.
  * 
  * Gets the system time and date in human readable values (UTC).
  **/
@@ -560,7 +562,7 @@ oobs_time_config_set_utc_time (OobsTimeConfig *config,
  * 
  * Returns the timezone set for #config.
  * 
- * Return Value: A pointer to the timezone as a string.
+ * Return Value: A pointer to the timezone as a string or #NULL.
  *               This string must not be freed, modified or stored.
  **/
 G_CONST_RETURN gchar*
@@ -570,7 +572,7 @@ oobs_time_config_get_timezone (OobsTimeConfig *config)
 
   g_return_val_if_fail (OOBS_IS_TIME_CONFIG (config), NULL);
 
-  priv = OOBS_TIME_CONFIG_GET_PRIVATE (config);
+  priv = config->_priv;
 
   return priv->timezone;
 }
